local core = LibStub("AceAddon-3.0"):GetAddon("YurrCombatLog")
local optionsModule = core:NewModule("YurrCombatLogOptions")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("YurrCombatLog", true)
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local frames = {}
local db

for i=1,7 do
	frames[format("%s%d", "ChatFrame", i)] = format("%s%d", "ChatFrame", i)
end

local colors = {
	-- unitColors
	{ id = 1297, name = L.self },
	{ id = 4991, name = PET },
	{ id = 1394, name = PARTY },
	{ id = 1396, name = RAID },
	{ id = 9087, name = L.guardian },
	{ id = 131072, name = FOCUS },
	{ id = 20334, name = L.enemies },
	{ id = 3864, name = L.outsider },
	-- schools
	{ id = SCHOOL_MASK_PHYSICAL, name = STRING_SCHOOL_PHYSICAL },
	{ id = SCHOOL_MASK_HOLY, name = STRING_SCHOOL_HOLY },
	{ id = SCHOOL_MASK_FIRE, name = STRING_SCHOOL_FIRE },
	{ id = SCHOOL_MASK_NATURE, name = STRING_SCHOOL_NATURE },
	{ id = SCHOOL_MASK_FROST, name = STRING_SCHOOL_FROST },
	{ id = SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SHADOW },
	{ id = SCHOOL_MASK_ARCANE, name = STRING_SCHOOL_ARCANE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE, name = STRING_SCHOOL_FLAMESTRIKE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST, name = STRING_SCHOOL_FROSTSTRIKE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE, name = STRING_SCHOOL_SPELLSTRIKE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE, name = STRING_SCHOOL_STORMSTRIKE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SHADOWSTRIKE },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_HOLYSTRIKE },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST, name = STRING_SCHOOL_FROSTFIRE },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE, name = STRING_SCHOOL_SPELLFIRE },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE, name = STRING_SCHOOL_FIRESTORM },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SHADOWFLAME },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_HOLYFIRE },
	{ id = SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE, name = STRING_SCHOOL_SPELLFROST },
	{ id = SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE, name = STRING_SCHOOL_FROSTSTORM },
	{ id = SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SHADOWFROST },
	{ id = SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_HOLYFROST },
	{ id = SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE, name = STRING_SCHOOL_SPELLSTORM },
	{ id = SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SPELLSHADOW },
	{ id = SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_DIVINE },
	{ id = SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_SHADOWSTORM },
	{ id = SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_HOLYSTORM },
	{ id = SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_SHADOWLIGHT },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE, name = STRING_SCHOOL_ELEMENTAL },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW, name = STRING_SCHOOL_CHROMATIC },
	{ id = SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_MAGIC },
	{ id = SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY, name = STRING_SCHOOL_CHAOS },
	-- otherColors
	{ id = 140000, name = "heal" },
	{ id = 140001, name = "miss" },
	{ id = 140002, name = "buff" },
	{ id = 140003, name = "debuff" },
	{ id = 140004, name = "skill" },
	{ id = 140005, name = "timestampColor" }
}

-- Ninjad from Chatter/Prat
local FORMATS = {
	[ADDON_DISABLED] = ADDON_DISABLED,
	["%I:%M:%S %p"] = L["%I:%M:%S %p"],
	["%I:%M:%S"] = L["%I:%M:%S"],
	["%X"] = L["%X"],
	["%I:%M"] = L["%I:%M"],
	["%H:%M"] = L["%H:%M"],
	["%M:%S"] = L["%M:%S"]
}
local COMBATLOG_EVENT_LIST = {
	"ENVIRONMENTAL_DAMAGE",
	"SWING_DAMAGE",
	"SWING_MISSED",
	"RANGE_DAMAGE",
	"RANGE_MISSED",
	"SPELL_CAST_START",
	"SPELL_CAST_SUCCESS",
	"SPELL_CAST_FAILED",
	"SPELL_MISSED",
	"SPELL_DAMAGE",
	"SPELL_HEAL",
	"SPELL_ENERGIZE",
	"SPELL_DRAIN",
	"SPELL_LEECH",
	"SPELL_SUMMON",
	"SPELL_RESURRECT",
	"SPELL_CREATE",
	"SPELL_INSTAKILL",
	"SPELL_INTERRUPT",
	"SPELL_EXTRA_ATTACKS",
	"SPELL_DURABILITY_DAMAGE",
	"SPELL_DURABILITY_DAMAGE_ALL",
	"SPELL_AURA_APPLIED",
	"SPELL_AURA_APPLIED_DOSE",
	"SPELL_AURA_REMOVED",
	"SPELL_AURA_REMOVED_DOSE",
	"SPELL_AURA_BROKEN",
	"SPELL_AURA_BROKEN_SPELL",
	"SPELL_AURA_REFRESH",
	"SPELL_DISPEL",
	"SPELL_STOLEN",
	"ENCHANT_APPLIED",
	"ENCHANT_REMOVED",
	"SPELL_PERIODIC_MISSED",
	"SPELL_PERIODIC_DAMAGE",
	"SPELL_PERIODIC_HEAL",
	"SPELL_PERIODIC_ENERGIZE",
	"SPELL_PERIODIC_DRAIN",
	"SPELL_PERIODIC_LEECH",
	"SPELL_DISPEL_FAILED",
	"DAMAGE_SHIELD",
	"DAMAGE_SHIELD_MISSED",
	"DAMAGE_SPLIT",
	"PARTY_KILL",
	"UNIT_DIED",
	"UNIT_DESTROYED",
	"UNIT_DISSIPATES",
	"SPELL_BUILDING_DAMAGE",
	"SPELL_BUILDING_HEAL",
};
local UNITFLAG_LIST = {
	--Controller affiliation
	"COMBATLOG_OBJECT_AFFILIATION_MINE",
	"COMBATLOG_OBJECT_AFFILIATION_PARTY",
	"COMBATLOG_OBJECT_AFFILIATION_RAID",
	"COMBATLOG_OBJECT_AFFILIATION_OUTSIDER",
	--Reaction
	"COMBATLOG_OBJECT_REACTION_FRIENDLY",
	"COMBATLOG_OBJECT_REACTION_NEUTRAL",
	"COMBATLOG_OBJECT_REACTION_HOSTILE",
	--Controller
	"COMBATLOG_OBJECT_CONTROL_PLAYER",
	"COMBATLOG_OBJECT_CONTROL_NPC",
	--Type
	"COMBATLOG_OBJECT_TYPE_PLAYER",
	"COMBATLOG_OBJECT_TYPE_NPC",
	"COMBATLOG_OBJECT_TYPE_PET",
	"COMBATLOG_OBJECT_TYPE_GUARDIAN",
	"COMBATLOG_OBJECT_TYPE_OBJECT",
	--Special cases (non-exclusive)
	"COMBATLOG_OBJECT_TARGET",
	"COMBATLOG_OBJECT_FOCUS",
	"COMBATLOG_OBJECT_MAINTANK",
	"COMBATLOG_OBJECT_MAINASSIST",
	"COMBATLOG_OBJECT_RAIDTARGET1",
	"COMBATLOG_OBJECT_RAIDTARGET2",
	"COMBATLOG_OBJECT_RAIDTARGET3",
	"COMBATLOG_OBJECT_RAIDTARGET4",
	"COMBATLOG_OBJECT_RAIDTARGET5",
	"COMBATLOG_OBJECT_RAIDTARGET6",
	"COMBATLOG_OBJECT_RAIDTARGET7",
	"COMBATLOG_OBJECT_RAIDTARGET8",
	"COMBATLOG_OBJECT_NONE",
}

local function createEvents() 
	local o = 1
	local t = {
		type = "group",
		name = L.events,
		order = 1,
		args = {},
	}
	
	for k, v in ipairs(COMBATLOG_EVENT_LIST) do
		t.args[v] = {
			type = "toggle",
			name = L[v],
			order = o,
			get = "GetEvent",
			set = "SetEvent",
		}
		if v == "SPELL_PERIODIC_ENERGIZE" or v == "SPELL_PERIODIC_DRAIN" or v == "SPELL_PERIODIC_LEECH" then
			t.args[v].desc = L[format("%s%s", "desc_", v)]
		end
		o = o + 1
	end
	return t
end
local function HexToRGBPerc(hex)
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
end
local function RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end
local function createColors()
	local t = {}
	for k,v in ipairs(colors) do
		local header = "unitColors"
		local headerline = 0
		local id = v.id
		if k >= 9 then
			if v.id >= 140000 then
				header = "otherColors"
				headerline = v.id
				id = v.name
			else
				header = "spellColors"
				headerline = k
			end
		end
		if k == 1 or k == 9 or v.id == 140000 then
			t[header] = {
				type = "header",
				name = L[header],
				order = headerline
			}
		end
		
		if k < 9 then k = k + 1
		elseif k > 8 and v.id < 140000 then k = k + 2
		elseif v.id >= 140000 then k = k + v.id
		end
		
		t[v.name] = {
			type = "color",
			name = v.name,
			order = k,
			get = function() return HexToRGBPerc(db.profile[header][id]) end,
			set = function(self,r,g,b) db.profile[header][id] = RGBPercToHex(r,g,b); core:Blizzard_CombatLog_Refilter() end,
		}
	end
	return t
end

function optionsModule:createWatch(k, v)
	local t = {
		type = "execute",
		name = select(6, GetPlayerInfoByGUID(k)),
		desc = L.clickToDelete,
		func = function(self, value)
			db.profile.watches[k] = nil
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.watches.args[k] = nil
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	return t
end
local function createWatches()
	local t = {}
	t.new = {
		type = "input",
		name = L.addWatch,
		order = 0,
		set = function(self,value)
			db.profile.watches[value] = UnitGUID(value)
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.watches.args[value] = optionsModule:createWatch(value, UnitGUID(value))
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	for k,v in pairs(db.profile.watches) do
		t[k] = optionsModule:createWatch(k, v)
	end
	return t
end

local function blacklistSpell(spellName)
	local t = {
		type = "execute",
		name = spellName,
		desc = L.clickToDeleteSpell,
		func = function(self, value)
			db.profile.blacklist[spellName] = nil
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.blacklist.args[spellName] = nil
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	return t
end
local function createBlacklist()
	local t = {}
	t.new = {
		type = "input",
		name = L.blackListSpell,
		order = 0,
		set = function(self,value)
			db.profile.blacklist[value] = value
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.blacklist.args[value] = blacklistSpell(value)
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	for spellName in pairs(db.profile.blacklist) do
		t[spellName] = blacklistSpell(spellName)
	end
	return t
end

local function whitelistSpell(spellName)
	local t = {
		type = "execute",
		name = spellName,
		desc = L.clickToDeleteSpell,
		func = function(self, value)
			db.profile.whitelist[spellName] = nil
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.whitelist.args[spellName] = nil
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	return t
end
local function createWhitelist()
	local t = {}
	t.new = {
		type = "input",
		name = L.whitelistSpell,
		order = 0,
		set = function(self,value)
			db.profile.whitelist[value] = value
			local options = AceConfigRegistry:GetOptionsTable("YurrCombatLog", "dialog", "YurrCombatLog-0.6")
			options.args.whitelist.args[value] = whitelistSpell(value)
			AceConfigRegistry:NotifyChange("YurrCombatLog")
			core:setFilter()
		end,
	}
	t.toggle = {
		type = "toggle",
		name = L.EnableWhitelist,
		order = 0,
		get = function() return db.profile.enableWhitelist end,
		set = function(info, value) db.profile.enableWhitelist = value end,
	}
	for spellName in pairs(db.profile.whitelist) do
		t[spellName] = whitelistSpell(spellName)
	end
	return t
end

local function createFormatting()
	local unitColors = db.profile.unitColors
	local spellColors = db.profile.spellColors
	local otherColors = db.profile.otherColors
	local braceColors = db.profile.braceColors
	local braces = db.profile.braces
	local t = {}
	local bracesStrings = {
		"Timestamp",
		"Source",
		"Spell",
		"Dest",
		"Amount",
		"AmountDots",
		"SchoolText"
	}
	t.apply = {
		type = "execute",
		name = APPLY,
		order = 1000,
		width = "full",
		func = function() core:setFilter() end,
	}
	t.general = {
		type = "header",
		name = GENERAL,
		order = 1,
	}
	
	t.timestamps = {
		type = "select",
		name = L.Timestamp,
		order = 2,
		values = {
			format("|cFF%s%s|r|cFF%s14:43:36|r|cFF%s%s|r", braceColors.Timestamp, braces.preTimestamp, otherColors.timestampColor, braceColors.Timestamp, braces.postTimestamp),
			format("|cFF%s%s|r14:43:36|cFF%s%s|r", braceColors.Timestamp, braces.preTimestamp, braceColors.Timestamp, braces.postTimestamp),
			format("%s|cFF%s14:43:36|r%s", braces.preTimestamp, otherColors.timestampColor, braces.postTimestamp),
			format("%s14:43:36%s", braces.preTimestamp, braces.postTimestamp),
			"14:43:36",
			format("|cFF%s14:43:36|r", otherColors.timestampColor),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorTimestamp = true
				db.profile.braceTimestamp = true
				db.profile.colorTimestampBrace = true
			elseif value == 2 then
				db.profile.colorTimestamp = false
				db.profile.braceTimestamp = true
				db.profile.colorTimestampBrace = true
			elseif value == 3 then
				db.profile.colorTimestamp = true
				db.profile.braceTimestamp = true
				db.profile.colorTimestampBrace = false
			elseif value == 4 then
				db.profile.colorTimestamp = false
				db.profile.braceTimestamp = true
				db.profile.colorTimestampBrace = false
			elseif value == 5 then
				db.profile.colorTimestamp = false
				db.profile.braceTimestamp = false
				db.profile.colorTimestampBrace = false
			elseif value == 6 then
				db.profile.colorTimestamp = true
				db.profile.braceTimestamp = false
				db.profile.colorTimestampBrace = false
			end
		end,
		get = function()
			if db.profile.colorTimestamp and db.profile.braceTimestamp and db.profile.colorTimestampBrace then
				return 1
			elseif not db.profile.colorTimestamp and db.profile.braceTimestamp and db.profile.colorTimestampBrace then
				return 2
			elseif db.profile.colorTimestamp and db.profile.braceTimestamp and not db.profile.colorTimestampBrace then
				return 3
			elseif not db.profile.colorTimestamp and db.profile.braceTimestamp and not db.profile.colorTimestampBrace then
				return 4
			elseif not db.profile.colorTimestamp and not db.profile.braceTimestamp and not db.profile.colorTimestampBrace then
				return 5
			elseif db.profile.colorTimestamp and not db.profile.braceTimestamp and not db.profile.colorTimestampBrace then
				return 6
			end
		end,
	}
	
	t.sourceUnit = {
		type = "select",
		name = L.Source,
		order = 3,
		width = "half",
		values = {
			format("|cFF%s%s|r|cFF%s%s|r|cFF%s%s|r", braceColors.Source, braces.preSource, unitColors[1297], YOU, braceColors.Source, braces.postSource),
			format("|cFF%s%s|r%s|cFF%s%s|r", braceColors.Source, braces.preSource, YOU, braceColors.Source, braces.postSource),
			format("%s|cFF%s%s|r%s", braces.preSource, unitColors[1297], YOU, braces.postSource),
			format("%s%s%s", braces.preSource, YOU, braces.postSource),
			YOU,
			format("|cFF%s%s|r", unitColors[1297], YOU),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorSrc = true
				db.profile.braceSrcUnit = true
				db.profile.colorSrcBraces = true
			elseif value == 2 then
				db.profile.colorSrc = false
				db.profile.braceSrcUnit = true
				db.profile.colorSrcBraces = true
			elseif value == 3 then
				db.profile.colorSrc = true
				db.profile.braceSrcUnit = true
				db.profile.colorSrcBraces = false
			elseif value == 4 then
				db.profile.colorSrc = false
				db.profile.braceSrcUnit = true
				db.profile.colorSrcBraces = false
			elseif value == 5 then
				db.profile.colorSrc = false
				db.profile.braceSrcUnit = false
				db.profile.colorSrcBraces = false
			elseif value == 6 then
				db.profile.colorSrc = true
				db.profile.braceSrcUnit = false
				db.profile.colorSrcBraces = false
			end
		end,
		get = function()
			if db.profile.colorSrc and db.profile.braceSrcUnit and db.profile.colorSrcBraces then
				return 1
			elseif not db.profile.colorSrc and db.profile.braceSrcUnit and db.profile.colorSrcBraces then
				return 2
			elseif db.profile.colorSrc and db.profile.braceSrcUnit and not db.profile.colorSrcBraces then
				return 3
			elseif not db.profile.colorSrc and db.profile.braceSrcUnit and not db.profile.colorSrcBraces then
				return 4
			elseif not db.profile.colorSrc and not db.profile.braceSrcUnit and not db.profile.colorSrcBraces then
				return 5
			elseif db.profile.colorSrc and not db.profile.braceSrcUnit and not db.profile.colorSrcBraces then
				return 6
			end
		end,
	}
	
	t.prefix = {
		type = "select",
		name = L.Prefix,
		order = 4,
		width = "half",
		values = {
			format("|cFF%s%s|r", spellColors[4], L.casts),
			"casts",
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorPrefix = true
			elseif value == 2 then
				db.profile.colorPrefix = false
			end
		end,
		get = function()
			if db.profile.colorPrefix then
				return 1
			elseif not db.profile.colorPrefix then
				return 2
			end
		end,
	}
	
	t.spell = {
		type = "select",
		name = L.Spell,
		order = 5,
		values = {
			format("|cFF%s%s|r|cFF%s%s|r|cFF%s|r|cFF%s%s", braceColors.Spell, braces.preSpell, spellColors[4], EXAMPLE_SPELL_FIREBALL, braceColors.Spell, braceColors.Spell, braces.postSpell),
			format("|cFF%s%s|r%s|cFF%s|r|cFF%s%s", braceColors.Spell, braces.preSpell, EXAMPLE_SPELL_FIREBALL, braceColors.Spell, braceColors.Spell, braces.postSpell),
			format("%s|cFF%s%s|r%s", braces.preSpell, spellColors[4], EXAMPLE_SPELL_FIREBALL, braces.postSpell),
			format("%s%s%s", braces.preSpell, EXAMPLE_SPELL_FIREBALL, braces.postSpell),
			EXAMPLE_SPELL_FIREBALL,
			format("|cFF%s%s|r", spellColors[4], EXAMPLE_SPELL_FIREBALL),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorSpell = true
				db.profile.braceSpell = true
				db.profile.colorSpellBraces = true
			elseif value == 2 then
				db.profile.colorSpell = false
				db.profile.braceSpell = true
				db.profile.colorSpellBraces = true
			elseif value == 3 then
				db.profile.colorSpell = true
				db.profile.braceSpell = true
				db.profile.colorSpellBraces = false
			elseif value == 4 then
				db.profile.colorSpell = false
				db.profile.braceSpell = true
				db.profile.colorSpellBraces = false
			elseif value == 5 then
				db.profile.colorSpell = false
				db.profile.braceSpell = false
				db.profile.colorSpellBraces = false
			elseif value == 6 then
				db.profile.colorSpell = true
				db.profile.braceSpell = false
				db.profile.colorSpellBraces = false
			end
		end,
		get = function()
			if db.profile.colorSpell and db.profile.braceSpell and db.profile.colorSpellBraces then
				return 1
			elseif not db.profile.colorSpell and db.profile.braceSpell and db.profile.colorSpellBraces then
				return 2
			elseif db.profile.colorSpell and db.profile.braceSpell and not db.profile.colorSpellBraces then
				return 3
			elseif not db.profile.colorSpell and db.profile.braceSpell and not db.profile.colorSpellBraces then
				return 4
			elseif not db.profile.colorSpell and not db.profile.braceSpell and not db.profile.colorSpellBraces then
				return 5
			elseif db.profile.colorSpell and not db.profile.braceSpell and not db.profile.colorSpellBraces then
				return 6
			end
		end,
	}
	
	t.suffix = {
		type = "select",
		name = L.suffix,
		order = 6,
		width = "half",
		values = {
			format("|cFF%s%s|r", otherColors.miss, strlower(MISS)),
			strlower(MISS),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorSuffix = true
			elseif value == 2 then
				db.profile.colorSuffix = false
			end
		end,
		get = function()
			if db.profile.colorSuffix then
				return 1
			elseif not db.profile.colorSuffix then
				return 2
			end
		end,
	}
	
	t.destUnit = {
		type = "select",
		name = L.Destination,
		order = 7,
		width = "half",
		values = {
			format("|cFF%s%s|r|cFF%s%s|r|cFF%s%s|r", braceColors.Dest, braces.preDest, unitColors[1297], YOU, braceColors.Dest, braces.postDest),
			format("|cFF%s%s|r%s|cFF%s%s|r", braceColors.Dest, braces.preDest, YOU, braceColors.Dest, braces.postDest),
			format("%s|cFF%s%s|r%s", braces.preDest, unitColors[1297], YOU, braces.postDest),
			format("%s%s%s", braces.preDest, YOU, braces.postDest),
			YOU,
			format("|cFF%s%s|r", unitColors[1297], YOU),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorDest = true
				db.profile.braceDestUnit = true
				db.profile.colorDestBraces = true
			elseif value == 2 then
				db.profile.colorDest = false
				db.profile.braceDestUnit = true
				db.profile.colorDestBraces = true
			elseif value == 3 then
				db.profile.colorDest = true
				db.profile.braceDestUnit = true
				db.profile.colorDestBraces = false
			elseif value == 4 then
				db.profile.colorDest = false
				db.profile.braceDestUnit = true
				db.profile.colorDestBraces = false
			elseif value == 5 then
				db.profile.colorDest = false
				db.profile.braceDestUnit = false
				db.profile.colorDestBraces = false
			elseif value == 6 then
				db.profile.colorDest = true
				db.profile.braceDestUnit = false
				db.profile.colorDestBraces = false
			end
		end,
		get = function()
			if db.profile.colorDest and db.profile.braceDestUnit and db.profile.colorDestBraces then
				return 1
			elseif not db.profile.colorDest and db.profile.braceDestUnit and db.profile.colorDestBraces then
				return 2
			elseif db.profile.colorDest and db.profile.braceDestUnit and not db.profile.colorDestBraces then
				return 3
			elseif not db.profile.colorDest and db.profile.braceDestUnit and not db.profile.colorDestBraces then
				return 4
			elseif not db.profile.colorDest and not db.profile.braceDestUnit and not db.profile.colorDestBraces then
				return 5
			elseif db.profile.colorDest and not db.profile.braceDestUnit and not db.profile.colorDestBraces then
				return 6
			end
		end,
	}
	
	t.AmountString = {
		type = "select",
		name = L.Amount,
		order = 8,
		width = "half",
		values = {
			format("|cFF%s%s|r|cFF%s936|r|cFF%s%s|r", braceColors.Amount, braces.preAmount, spellColors[4], braceColors.Amount, braces.postAmount),
			format("|cFF%s%s|r936|cFF%s%s|r", braceColors.Amount, braces.preAmount, braceColors.Amount, braces.postAmount),
			format("%s|cFF%s936|r%s", braces.preAmount, spellColors[4], braces.postAmount),
			format("%s936%s", braces.preAmount, braces.postAmount),
			"936",
			format("|cFF%s936|r", spellColors[4]),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorAmount = true
				db.profile.braceAmount = true
				db.profile.colorAmountBraces = true
			elseif value == 2 then
				db.profile.colorAmount = false
				db.profile.braceAmount = true
				db.profile.colorAmountBraces = true
			elseif value == 3 then
				db.profile.colorAmount = true
				db.profile.braceAmount = true
				db.profile.colorAmountBraces = false
			elseif value == 4 then
				db.profile.colorAmount = false
				db.profile.braceAmount = true
				db.profile.colorAmountBraces = false
			elseif value == 5 then
				db.profile.colorAmount = false
				db.profile.braceAmount = false
				db.profile.colorAmountBraces = false
			elseif value == 6 then
				db.profile.colorAmount = true
				db.profile.braceAmount = false
				db.profile.colorAmountBraces = false
			end
		end,
		get = function()
			if db.profile.colorAmount and db.profile.braceAmount and db.profile.colorAmountBraces then
				return 1
			elseif not db.profile.colorAmount and db.profile.braceAmount and db.profile.colorAmountBraces then
				return 2
			elseif db.profile.colorAmount and db.profile.braceAmount and not db.profile.colorAmountBraces then
				return 3
			elseif not db.profile.colorAmount and db.profile.braceAmount and not db.profile.colorAmountBraces then
				return 4
			elseif not db.profile.colorAmount and not db.profile.braceAmount and not db.profile.colorAmountBraces then
				return 5
			elseif db.profile.colorAmount and not db.profile.braceAmount and not db.profile.colorAmountBraces then
				return 6
			end
		end,
	}
	
	t.AmountDotString = {
		type = "select",
		name = L.AmountDots,
		order = 8,
		width = "half",
		values = {
			format("|cFF%s%s|r|cFF%s468|r|cFF%s%s|r", braceColors.AmountDots, braces.preAmountDots, spellColors[4], braceColors.AmountDots, braces.postAmountDots),
			format("%s|cFF%s468|r%s", braces.preAmountDots, spellColors[4], braces.postAmountDots),
			format("|cFF%s468|r", spellColors[4]),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.colorAmountDotBraces = true
				db.profile.braceAmountDots = true
			elseif value == 2 then
				db.profile.colorAmountDotBraces = false
				db.profile.braceAmountDots = true
			elseif value == 3 then
				db.profile.colorAmountDotBraces = false
				db.profile.braceAmountDots = false
			end
		end,
		get = function()
			if db.profile.colorAmountDotBraces and db.profile.braceAmountDots then
				return 1
			elseif not db.profile.colorAmountDotBraces and db.profile.braceAmountDots then
				return 2
			elseif not db.profile.colorAmountDotBraces and not db.profile.braceAmountDots then
				return 3
			end
		end,
	}
	
	t.crits = {
		type = "select",
		name = L.Crit,
		order = 9,
		width = "half",
		values = {
			format("%s|cFF%s1872|r%s", braces.preCrit, spellColors[4], braces.postCrit),
			format("|cFF%s1872|r", spellColors[4]),
		},
		set = function(info, value)
			if value == 1 then
				db.profile.markCrits = true
			elseif value == 2 then
				db.profile.markCrits = false
			end
		end,
		get = function()
			if db.profile.markCrits then
				return 1
			elseif not db.profile.markCrits then
				return 2
			end
		end,
	}
	
	t.overhealingoverkill = {
		type = "select",
		name = format("%s/%s", L.overhealing, L.overkill),
		order = 9,
		values = {
			format("%d(+%d)", 312, 624),
			936,
		},
		set = function(info, value)
			if value == 1 then
				db.profile.overhealing = true
				db.profile.overkill = true
			elseif value == 2 then
				db.profile.overhealing = false
				db.profile.overkill = false
			end
		end,
		get = function()
			if db.profile.overhealing and db.profile.overkill then
				return 1
			elseif not db.profile.overhealing and not db.profile.overkill then
				return 2
			end
		end,
	}
	
	t.resultString = {
		type = "toggle",
		name = L.ShowResultString,
		desc = L.Desc_ResultString,
		order = 10,
		get = function() return db.profile.resultString end,
		set = function(info, value) db.profile.resultString = value end,
	}
	
	t.powerType = {
		type = "toggle",
		name = L.ShowPowerType,
		desc = L.Desc_PowerTypes,
		order = 11,
		get = function() return db.profile.powerType end,
		set = function(info, value) db.profile.powerType = value end,
	}
	
	t.schoolText = {
		type = "select",
		name = L.SchoolText,
		order = 12,
		width = "half",
		values = {
			format("|cFF%s%s|r|cFF%s%s|r|cFF%s%s|r", braceColors.Source, braces.preSchoolText, spellColors[4], RESISTANCE_TYPE2, braceColors.Source, braces.postSchoolText),
			format("|cFF%s%s|r%s|cFF%s%s|r", braceColors.Source, braces.preSchoolText, RESISTANCE_TYPE2, braceColors.Source, braces.postSchoolText),
			format("%s|cFF%s%s|r%s", braces.preSchoolText, spellColors[4], RESISTANCE_TYPE2, braces.postSchoolText),
			format("%s%s%s", braces.preSchoolText, RESISTANCE_TYPE2, braces.postSchoolText),
			RESISTANCE_TYPE2,
			format("|cFF%s%s|r", spellColors[4], RESISTANCE_TYPE2),
			DISABLE,
		},
		set = function(info, value)
			if value == 7 then
				db.profile.schoolText = false
			elseif value == 1 then
				db.profile.colorSchoolText = true
				db.profile.braceSchoolText = true
				db.profile.colorSchoolTextBraces = true
				db.profile.schoolText = true
			elseif value == 2 then
				db.profile.colorSchoolText = false
				db.profile.braceSchoolText = true
				db.profile.colorSchoolTextBraces = true
				db.profile.schoolText = true
			elseif value == 3 then
				db.profile.colorSchoolText = true
				db.profile.braceSchoolText = true
				db.profile.colorSchoolTextBraces = false
				db.profile.schoolText = true
			elseif value == 4 then
				db.profile.colorSchoolText = false
				db.profile.braceSchoolText = true
				db.profile.colorSchoolTextBraces = false
				db.profile.schoolText = true
			elseif value == 5 then
				db.profile.colorSchoolText = false
				db.profile.braceSchoolText = false
				db.profile.colorSchoolTextBraces = false
				db.profile.schoolText = true
			elseif value == 6 then
				db.profile.colorSchoolText = true
				db.profile.braceSchoolText = false
				db.profile.colorSchoolTextBraces = false
				db.profile.schoolText = true
			end
		end,
		get = function()
			if not db.profile.schoolText then
				return 7
			elseif db.profile.colorSchoolText and db.profile.braceSchoolText and db.profile.colorSchoolTextBraces then
				return 1
			elseif not db.profile.colorSchoolText and db.profile.braceSchoolText and db.profile.colorSchoolTextBraces then
				return 2
			elseif db.profile.colorSchoolText and db.profile.braceSchoolText and not db.profile.colorSchoolTextBraces then
				return 3
			elseif not db.profile.colorSchoolText and db.profile.braceSchoolText and not db.profile.colorSchoolTextBraces then
				return 4
			elseif not db.profile.colorSchoolText and not db.profile.braceSchoolText and not db.profile.colorSchoolTextBraces then
				return 5
			elseif db.profile.colorSchoolText and not db.profile.braceSchoolText and not db.profile.colorSchoolTextBraces then
				return 6
			end
		end,
	}
	
	t.braces = {
		type = "header",
		name = L.braces,
		order = 40,
	}
	for k,v in pairs(bracesStrings) do
		t[v] = {
			type = "input",
			name = L[v],
			width = "half",
			order = 41,
			get = function() return format("%s|%s", db.profile.braces[format("pre%s", v)], db.profile.braces[format("post%s", v)]) end,
			set = function(info, value)
				local pre, post = strsplit("|", value)
				db.profile.braces[format("pre%s", v)] = pre
				db.profile.braces[format("post%s", v)] = post
			end,
		}
	end
	
	t.bracesColors = {
		order = 60,
		type = "header",
		name = L.braceColors,
	}
	for k,v in pairs(db.profile.braceColors) do
		t[format("braceColors%s", k)] = {
			type = "color",
			name = L[k],
			order = 61,
			get = function() return HexToRGBPerc(db.profile.braceColors[k]) end,
			set = function(info, r,g,b) db.profile.braceColors[k] = RGBPercToHex(r,g,b) end,
		}
	end
	
	return t
end

local filter = 1

local function getYCLFlagsTable()
	return {
		Affiliation = {
			type = "header",
			name = "Affiliation",
			order = 1,
		},
		COMBATLOG_OBJECT_AFFILIATION_MINE = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_AFFILIATION_MINE,
			order = 2,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_AFFILIATION_PARTY = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_AFFILIATION_PARTY,
			order = 3,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_AFFILIATION_RAID = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_AFFILIATION_RAID,
			order = 4,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_AFFILIATION_OUTSIDER,
			order = 5,
			get = "GetFlags",
			set = "SetFlags",
		},
		Reaction = {
			type = "header",
			name = "Reaction",
			order = 6,
		},
		COMBATLOG_OBJECT_REACTION_FRIENDLY = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_REACTION_FRIENDLY,
			order = 7,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_REACTION_NEUTRAL = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_REACTION_NEUTRAL,
			order = 8,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_REACTION_HOSTILE = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_REACTION_HOSTILE,
			order = 9,
			get = "GetFlags",
			set = "SetFlags",
		},
		Control = {
			type = "header",
			name = "Control",
			order = 10,
		},
		COMBATLOG_OBJECT_CONTROL_PLAYER = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_CONTROL_PLAYER,
			order = 11,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_CONTROL_NPC = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_CONTROL_NPC,
			order = 12,
			get = "GetFlags",
			set = "SetFlags",
		},
		Type = {
			type = "header",
			name = "Type",
			order = 13,
		},
		COMBATLOG_OBJECT_TYPE_PLAYER = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TYPE_PLAYER,
			order = 14,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_TYPE_NPC = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TYPE_NPC,
			order = 15,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_TYPE_PET = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TYPE_PET,
			order = 16,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_TYPE_GUARDIAN = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TYPE_GUARDIAN,
			order = 17,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_TYPE_OBJECT = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TYPE_OBJECT,
			order = 18,
			get = "GetFlags",
			set = "SetFlags",
		},
		Special = {
			type = "header",
			name = "Special",
			order = 19,
		},
		COMBATLOG_OBJECT_TARGET = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_TARGET,
			order = 20,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_FOCUS = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_FOCUS,
			order = 21,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_MAINTANK = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_MAINTANK,
			order = 22,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_MAINASSIST = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_MAINASSIST,
			order = 23,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET1 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET1,
			order = 24,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET2 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET2,
			order = 25,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET3 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET3,
			order = 26,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET4 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET4,
			order = 27,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET5 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET5,
			order = 28,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET6 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET6,
			order = 29,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET7 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET7,
			order = 30,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_RAIDTARGET8 = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_RAIDTARGET8,
			order = 31,
			get = "GetFlags",
			set = "SetFlags",
		},
		COMBATLOG_OBJECT_NONE = {
			type = "toggle",
			name = L.COMBATLOG_OBJECT_NONE,
			order = 32,
			get = "GetFlags",
			set = "SetFlags",
		},
	}
end

local function getYCLOptionsTable()
	return {
		name = "YurrCombatLog",
		handler = core,
		type = "group",
		childGroups  = "tab",
		args = {
			general = {
				type = "group",
				name = GENERAL,
				order = 1,
				args = {
					outputFrame = {
						type = "select",
						name = L.outputFrame,
						order = 1,
						values = frames,
						get = function() return db.profile.outputFrame end,
						set = function(self, value)
							_G[db.profile.outputFrame]:Clear()
							db.profile.outputFrame = value
							core:Blizzard_CombatLog_Refilter()
							core:PositionFlyOut()
						end,
					},
					timestamp = {
						type = "select",
						name = L.Timestamp,
						order = 2,
						values = FORMATS,
						get = function() return db.profile.timestampFormat end,
						set = function(self, value) 
							if value == ADDON_DISABLED then
								db.profile.timestamp = false
							else
								db.profile.timestamp = true
							end
							db.profile.timestampFormat = value
							core:Blizzard_CombatLog_Refilter()
						end,
					},
					quickButtons = {
						type = "toggle",
						name = L.quickButtons,
						order = 3,
						get = function() return db.profile.quickButtons end,
						set = function(info, value) 
							if value then
								YCLFlyOut:Show()
								YCLFlyOut:SetFrameStrata("MEDIUM")
							else
								YCLFlyOut:Hide()
							end
							db.profile.quickButtons = value
						end,
					},
					contextMenu = {
						type = "toggle",
						name = L.contextMenu,
						order = 4,
						get = function() return db.profile.contextMenu end,
						set = function(info, value) 
							if value then
								core:InsertContext()
							else
								core:RemoveContext()
							end
							db.profile.contextMenu = value
						end,
					},
					colorByClass = {
						type = "toggle",
						name = L.colorByClass,
						order = 5,
						get = function() return db.profile.colorUnitByClass end,
						set = function(info, value)
							db.profile.colorUnitByClass = value
							core:Blizzard_CombatLog_Refilter()
						end,
					},
				},
			},
			filters = {
				type = "group",
				name = L.filters,
				order = 2,
				childGroups  = "tab",
				args = {
					filter = {
						type = "select",
						name = FILTER,
						order = 1,
						values = function()
							local t = {}
							for k, v in pairs(db.profile.filters) do
								if v then
									tinsert(t, k, db.profile.filters[k].name)
								end
							end
							return t
						end,
						get = function() return filter end,
						set = function(self, value) filter = value end,
					},
					new = {
						type = "input",
						name = L.addFilter,
						desc = L.desc_addFilter,
						order = 2,
						set = function(self,value)
							if value == "" then return end
							local filter = {}
							filter.name = value
							filter.active = false
							for k,v in ipairs(COMBATLOG_EVENT_LIST) do
								filter[v] = false
							end
							filter.sourceUnit = {}
							for k,v in ipairs(UNITFLAG_LIST) do
								filter.sourceUnit[v] = false
							end
							filter.destUnit = {}
							for k,v in ipairs(UNITFLAG_LIST) do
								filter.destUnit[v] = false
							end
							tinsert(db.profile.filters, filter)
							core:ReInitQuickButtons()
						end,
					},
					delete = {
						type = "execute",
						name = DELETE,
						order = 4,
						width = "half",
						func = function()
							db.profile.filters[filter] = "false"
							core:ReInitQuickButtons()
							filter = filter - 1
							-- hack to prevent a negative filter number
							if filter < 1 then
								for k,v in ipairs(db.profile.filters) do
									if v then
										filter = k
										break
									end
								end
							end
						end,
						confirm = function() return format("%s %s?",DELETE, db.profile.filters[filter].name) end,
					},
					refilter = {
						type = "execute",
						-- name = L.refilter,
						name = APPLY,
						order = 4,
						width = "half",
						func = function() core:setFilter() end,
					},
					active = {
						type = "toggle",
						name = L.active,
						order = 5,
						width = "half",
						get = function() return db.profile.filters[filter].active end,
						set = function(self, value) db.profile.filters[filter].active = value end,
					},
					events = createEvents(),
					sourceUnit = {
						type = "group",
						name = L.Source,
						order = 2,
						args = getYCLFlagsTable(),
					},
					destUnit = {
						type = "group",
						name = L.Dest,
						order = 3,
						args = getYCLFlagsTable(),
					},
				},
			},
			colors = {
				type = "group",
				name = COLORS,
				order = 3,
				args = createColors(),
			},
			formatting = {
				type = "group",
				name = "Format",
				order = 4,
				args = createFormatting(),
			},
			watches = {
				type = "group",
				name = "Watches",
				order = 5,
				args = createWatches(),
			},
			blacklist = {
				type = "group",
				name = L.blacklist,
				order = 6,
				args = createBlacklist(),
			},
			whitelist = {
				type = "group",
				name = L.whitelist,
				order = 7,
				args = createWhitelist(),
			},
			profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(core.db),
		}
	}
end

function optionsModule:createFilter(name)
	local t = {
		name = name,
		active = false,
		ENVIRONMENTAL_DAMAGE = true,
		SWING_DAMAGE = true,
		SWING_MISSED = true,
		RANGE_DAMAGE = true,
		RANGE_MISSED = true,
		SPELL_CAST_START = false,
		SPELL_CAST_SUCCESS = false,
		SPELL_CAST_FAILED = false,
		SPELL_MISSED = true,
		SPELL_DAMAGE = true,
		SPELL_HEAL = true,
		SPELL_ENERGIZE = true,
		SPELL_DRAIN = true,
		SPELL_LEECH = true,
		SPELL_SUMMON = true,
		SPELL_RESURRECT = true,
		SPELL_CREATE = true,
		SPELL_INSTAKILL = true,
		SPELL_INTERRUPT = true,
		SPELL_EXTRA_ATTACKS = true,
		SPELL_DURABILITY_DAMAGE = false,
		SPELL_DURABILITY_DAMAGE_ALL = false,
		SPELL_AURA_APPLIED = true,
		SPELL_AURA_APPLIED_DOSE = true,
		SPELL_AURA_REMOVED = true,
		SPELL_AURA_REMOVED_DOSE = true,
		SPELL_AURA_BROKEN = true,
		SPELL_AURA_BROKEN_SPELL = true,
		SPELL_AURA_REFRESH = true,
		SPELL_DISPEL = true,
		SPELL_STOLEN = true,
		ENCHANT_APPLIED = true,
		ENCHANT_REMOVED = true,
		SPELL_PERIODIC_MISSED = true,
		SPELL_PERIODIC_DAMAGE = true,
		SPELL_PERIODIC_HEAL = true,
		SPELL_PERIODIC_ENERGIZE = true,
		SPELL_PERIODIC_DRAIN = true,
		SPELL_PERIODIC_LEECH = true,
		SPELL_DISPEL_FAILED = true,
		DAMAGE_SHIELD = true,
		DAMAGE_SHIELD_MISSED = true,
		DAMAGE_SPLIT = true,
		PARTY_KILL = true,
		UNIT_DIED = true,
		UNIT_DESTROYED = true,
		UNIT_DISSIPATES = true,
		SPELL_BUILDING_DAMAGE = true,
		SPELL_BUILDING_HEAL = true,
		sourceUnit = {
			--Controller affiliation
			COMBATLOG_OBJECT_AFFILIATION_MINE = false,
			COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
			COMBATLOG_OBJECT_AFFILIATION_RAID = false,
			COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
			--Reaction
			COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
			COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
			COMBATLOG_OBJECT_REACTION_HOSTILE = false,
			--Controller
			COMBATLOG_OBJECT_CONTROL_PLAYER = false,
			COMBATLOG_OBJECT_CONTROL_NPC = false,
			--Type
			COMBATLOG_OBJECT_TYPE_PLAYER = false,
			COMBATLOG_OBJECT_TYPE_NPC = false,
			COMBATLOG_OBJECT_TYPE_PET = false,
			COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
			COMBATLOG_OBJECT_TYPE_OBJECT = false,
			--Special cases (non-exclusive)
			COMBATLOG_OBJECT_TARGET = false,
			COMBATLOG_OBJECT_FOCUS = false,
			COMBATLOG_OBJECT_MAINTANK = false,
			COMBATLOG_OBJECT_MAINASSIST = false,
			COMBATLOG_OBJECT_RAIDTARGET1 = false,
			COMBATLOG_OBJECT_RAIDTARGET2 = false,
			COMBATLOG_OBJECT_RAIDTARGET3 = false,
			COMBATLOG_OBJECT_RAIDTARGET4 = false,
			COMBATLOG_OBJECT_RAIDTARGET5 = false,
			COMBATLOG_OBJECT_RAIDTARGET6 = false,
			COMBATLOG_OBJECT_RAIDTARGET7 = false,
			COMBATLOG_OBJECT_RAIDTARGET8 = false,
			COMBATLOG_OBJECT_NONE = false,
		},
		destUnit = {
			--Controller affiliation
			COMBATLOG_OBJECT_AFFILIATION_MINE = false,
			COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
			COMBATLOG_OBJECT_AFFILIATION_RAID = false,
			COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
			--Reaction
			COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
			COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
			COMBATLOG_OBJECT_REACTION_HOSTILE = false,
			--Controller
			COMBATLOG_OBJECT_CONTROL_PLAYER = false,
			COMBATLOG_OBJECT_CONTROL_NPC = false,
			--Type
			COMBATLOG_OBJECT_TYPE_PLAYER = false,
			COMBATLOG_OBJECT_TYPE_NPC = false,
			COMBATLOG_OBJECT_TYPE_PET = false,
			COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
			COMBATLOG_OBJECT_TYPE_OBJECT = false,
			--Special cases (non-exclusive)
			COMBATLOG_OBJECT_TARGET = false,
			COMBATLOG_OBJECT_FOCUS = false,
			COMBATLOG_OBJECT_MAINTANK = false,
			COMBATLOG_OBJECT_MAINASSIST = false,
			COMBATLOG_OBJECT_RAIDTARGET1 = false,
			COMBATLOG_OBJECT_RAIDTARGET2 = false,
			COMBATLOG_OBJECT_RAIDTARGET3 = false,
			COMBATLOG_OBJECT_RAIDTARGET4 = false,
			COMBATLOG_OBJECT_RAIDTARGET5 = false,
			COMBATLOG_OBJECT_RAIDTARGET6 = false,
			COMBATLOG_OBJECT_RAIDTARGET7 = false,
			COMBATLOG_OBJECT_RAIDTARGET8 = false,
			COMBATLOG_OBJECT_NONE = false,
		},
	},
	tinsert(db.filters, t)
end

function core:SetEvent(self, value)
	db.profile.filters[filter][self[#self]] = value
end
function core:GetEvent(self)
	return db.profile.filters[filter][self[#self]]
end

function core:SetFlags(self, value)
	db.profile.filters[filter][self[#self-1]][self[#self]] = value
end
function core:GetFlags(self)
	return db.profile.filters[filter][self[#self-1]][self[#self]]
end

function optionsModule:OnInitialize()
	db = core.db;
	LibStub("AceConfig-3.0"):RegisterOptionsTable("YurrCombatLog", getYCLOptionsTable(), {"/yurrcombtlog", "/ycl"})
	ChatFrame1:AddMessage("Module enabled");
end